<?php


include 'config.php';


// functions to read/write the current post counter
function readPostCounter($jsonFilePostCounter)
{
    if (file_exists($jsonFilePostCounter)) {
        $fp = fopen($jsonFilePostCounter, 'r');
        if (flock($fp, LOCK_SH)) { //shared lock for reading
            $counter = json_decode(fread($fp, filesize($jsonFilePostCounter)), true) ?: 0;
            flock($fp, LOCK_UN); 
            fclose($fp);
            return $counter;
        }
    }
    return 0;
}
function writePostCounter($jsonFilePostCounter, $counter)
{
    $fp = fopen($jsonFilePostCounter, 'w');
    if (flock($fp, LOCK_EX)) { 
        fwrite($fp, json_encode($counter, JSON_PRETTY_PRINT));
        fflush($fp);
        flock($fp, LOCK_UN);
    }
    fclose($fp);
}


//read/write for jsons on filepath
function readJsonFile($filePath)
{
    if (file_exists($filePath)) {
        $fp = fopen($filePath, 'r');
        if (flock($fp, LOCK_SH)) {
            $data = json_decode(fread($fp, filesize($filePath)), true) ?: [];
            flock($fp, LOCK_UN);
            fclose($fp);
            return $data;
        }
    }
    return [];
}

function writeJsonFile($filePath, $data)
{
    $fp = fopen($filePath, 'w');
    if (flock($fp, LOCK_EX)) {
        fwrite($fp, json_encode($data, JSON_PRETTY_PRINT));
        fflush($fp);
        flock($fp, LOCK_UN);
    }
    fclose($fp);
}

// read messages and post counter based on the board id
$messages = readJsonFile($jsonFileMessages);
$postCounter = readJsonFile($jsonFilePostCounter);


// function to read users from JSON file
function readUsers($jsonFileUsers) {
    if (file_exists($jsonFileUsers)) {
        return json_decode(file_get_contents($jsonFileUsers), true) ?: [];
    } else { return []; }
}

// login processing
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['login'])) {
    if (isset($_POST['username']) && isset($_POST['password']) && isset($_POST['user_id'])) {
        $username = htmlspecialchars(trim($_POST['username']));
        $password = $_POST['password'];
        $userId = intval($_POST['user_id']);
        $users = readUsers($jsonFileUsers);
		
		$loggedInUser = null;

        foreach ($users as $user) {
            if (($user['user_id']) === ($userId) && ($user['username']) === ($username)) {
                if (password_verify($password, $user['password'])) {
                    $loggedInUser = $user;
					$_POST['username'] = $user['username'];
                    break;
                }
            }
        }
        if ($loggedInUser) {
            $_SESSION['user'] = $loggedInUser;
            header('Location: ' . $_SERVER['PHP_SELF'] . "?b=" . $boardId);
            exit;
        } else { echo "Invalid login."; }
    }
}

// handle logout
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['logout'])) {
    session_unset();//clear all session variables
    session_destroy();//destroy the session
    header('Location: ' . $_SERVER['PHP_SELF'] . "?b=" . $boardId);
    exit;
}


//functions to format message, process markup, etc.
function extractUrls($text)
{
    $urlPattern = '/\bhttps?:\/\/[^\s]+\b/i'; //typical url 
    preg_match_all($urlPattern, $text, $matches);
    return $matches[0];
}
//check various extensions
function isImageUrl($url)
{
    $imageExtensions = ['.jpg', '.jpeg', '.png', '.gif'];
    foreach ($imageExtensions as $ext) {
        if (stripos($url, $ext) !== false) { return true; }
    }
    return false;
}
function isAudioUrl($url)
{
    $audioExtensions = ['.ogg', '.wav', '.flac', '.mp3'];
    foreach ($audioExtensions as $ext) {
        if (stripos($url, $ext) !== false) { return true; }
    }
    return false;
}
function isVideoUrl($url)
{
    $videoExtensions = ['.mp4', '.webm', '.mov', '.avi'];//note, support ogg video
    foreach ($videoExtensions as $ext) {
        if (stripos($url, $ext) !== false) { return true; }
    }
    return false;
}
function isYouTubeUrl($url){return preg_match('/^(https?:\/\/)?(www\.)?(youtube\.com|youtu\.be)\/(watch\?v=[\w-]+|[\w-]+)/i', $url);}

//perform markup
function formatMessage($message) {
    global $config;


    //handle emotes with rotation
    $message = preg_replace_callback(
        '/::([a-zA-Z0-9_]+)(?::(\d+))?::/',
        function ($matches) use ($config) {
            $emoteName = $matches[1];
            $degrees = isset ($matches[2]) ? $matches[2] : 0;
            $rotationStyle = $degrees ? 'transform: rotate(' . $degrees . 'deg);' : '';
            return '<img src="' . $config['filePathSite'] . '/' . $config['filePathSiteEmotes'] . '/' . $emoteName . '.gif" alt="' . $emoteName . '" style="width:16px; height:16px; vertical-align:middle; ' . $rotationStyle . '">';
        },
        $message
    );


    //pre split markup
	$message = preg_replace('/\|\|(.*?)\|\|/', '<span class="spoiler">$1</span>', $message);
    $message = preg_replace('/\*(.*?)\*/', '*<i>$1</i>*', $message);
    $message = preg_replace('/```(.*?)```/', '```<span class="code">$1</span>```', $message);

    // url extraction
    $urls = extractUrls($message);
    foreach ($urls as $url) {
        if (isImageUrl($url)) {//embed image url
            $imgTag = '<img src="' . htmlspecialchars($url) . '" class="embedded-image">';
            $message = str_replace($url, $imgTag, $message);
        } elseif (isAudioUrl($url)) {
            $audioTag = '<audio controls> <source src="' . htmlspecialchars($url) . '" > </audio>';
            $message = str_replace($url, $audioTag, $message);
        } elseif (isVideoUrl($url)) {
            $audioTag = '<video controls> <source src="' . htmlspecialchars($url) . '" > </video>';
            $message = str_replace($url, $audioTag, $message);
        } elseif (isYouTubeUrl($url)) {
            $youtubeId = '';
            if (preg_match('/watch\?v=([\w-]+)/i', $url, $matches)) { $youtubeId = $matches[1];
            } elseif (preg_match('/youtu\.be\/([\w-]+)/i', $url, $matches)) { $youtubeId = $matches[1]; }
            if ($youtubeId) {
                $iframeTag = '<iframe width="560" height="315" src="https://www.youtube.com/embed/' . htmlspecialchars($youtubeId) . '" frameborder="0" allowfullscreen></iframe>';
                $message = str_replace($url, $iframeTag, $message);
            }
        } else {//ignore urls not pointing to a particular supported filetype
            $linkTag = '<a href="' . htmlspecialchars($url) . '" target="_blank">' . htmlspecialchars($url) . '</a>';
            $message = str_replace($url, $linkTag, $message);
        }
    }

    //split message into lines
    $lines = explode("\n", $message);
    $formattedMessage = '';



    //loop through each line
    foreach ($lines as $line) {//prefix markup
        if (preg_match('/#(\d+)/', $line, $matches)) { //mention post by number
            //$postNumber = intval($matches[1]);
            $formattedMessage .= '<span class="message-mention">#' . intval($matches[1]) . '</span>';
            $line = preg_replace('/#(\d+)/', '', $line);
            //redtext
        }

        if (strpos($line, '&gt;') === 0) {
            $formattedMessage .= '<span class="greentext">' . $line . '</span>';
		} elseif (strpos($line, '&lt;') === 0) {
			$formattedMessage .= '<span class="orangetext">' . $line . '</span>';
		} elseif (strpos($line, ',') === 0) {
			$formattedMessage .= '<span class="redtext">' . $line . '</span>';
		} elseif (strpos($line, '.') === 0) {
			$formattedMessage .= '<span class="bluetext">' . $line . '</span>';
        } else {$formattedMessage .= $line ; }
		$formattedMessage .= '<br>';
    }
    return $formattedMessage;
}

//if its a POST request, handle the form submission
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['message'])) {
        //$messages = readJsonFile($jsonFileMessages);

        $username = isset($_SESSION['user']['username']) ? htmlspecialchars($_SESSION['user']['username']) : 'ANON';
        if (trim($username) === '') { $username = 'ANON'; }

        $message = htmlspecialchars($_POST['message']);

        $user_id = $_SESSION['user']['user_id'];

        if (strlen($username) > 32) { echo "Username too long."; exit; }
        if (strlen($message) > 4096) { echo "Message too long."; exit; }
        if (strlen($message) == 0) { echo "No message."; exit; }

        $postCounter = readPostCounter($jsonFilePostCounter);
        $postCounter++;

        //message object
        $newMessage = [
            'post_number' => readPostCounter($jsonFilePostCounter) + 1,
            'timestamp' => date('Y-m-d H:i:s'),
            'username' => $username,
            'user_id' => $user_id,
            'message' => htmlspecialchars($_POST['message']),
            //'file_name' => $uploadedFileName
        ];

        //read existing messages, append the new message, and save back to the JSON file
        $messages = readJsonFile($jsonFileMessages);
        $messages[] = $newMessage;

        writeJsonFile($jsonFileMessages, $messages);

        writePostCounter($jsonFilePostCounter, $postCounter);

        //avoid resubmission on refresh
        header('Location: ' . $_SERVER['PHP_SELF'] . "?b=" . $boardId);
        exit;
    }
}

//if it's a GET request, display the chat, display the last x amount of lines
$messages = readJsonFile($jsonFileMessages);//show all
$messages = array_slice($messages, -3000);


?>