<?php
include 'config.php';

$uploadsFilePath = $config['uploadsFilePath'];
$jsonsFilePath = $config['jsonsFilePath'];
$uploadsUserProfilePath = $config['uploadsUserProfilePath'];

$jsonFileUsers = $jsonsFilePath . '/' . $config['jsonFileUsers'];
$jsonFileUserCounter = $jsonsFilePath . '/' . $config['jsonFileUserCounter'];

function readUserCounter($jsonFileUserCounter) {
    if (file_exists($jsonFileUserCounter)) {
        return json_decode(file_get_contents($jsonFileUserCounter), true) ?: 0;
    } else { return 0; }
}

function writeUserCounter($jsonFileUserCounter, $counter) {
    file_put_contents($jsonFileUserCounter, json_encode($counter, JSON_PRETTY_PRINT));
}

function readUsers($jsonFileUsers) {
    if (file_exists($jsonFileUsers)) {
        return json_decode(file_get_contents($jsonFileUsers), true) ?: [];
    } else { return []; }
}

function writeUsers($jsonFileUsers, $users) {
    file_put_contents($jsonFileUsers, json_encode($users, JSON_PRETTY_PRINT));
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['username']) && isset($_POST['password']) && isset($_FILES['pfp'])) {
        $username = trim(htmlspecialchars($_POST['username']));
        $password = $_POST['password'];
        
        if (strlen($username) > 32 || strlen($username) == 0) {
            echo "Invalid username.";
            exit;
        }

        //profile picture
        $pfp = $_FILES['pfp'];
        if ($pfp['size'] > 8192) {
            echo "PFP too large.";
            exit;
        }

        $allowedExtensions = ['gif'];
        $fileInfo = pathinfo($pfp['name']);
        $fileExtension = strtolower($fileInfo['extension']);

        if (!in_array($fileExtension, $allowedExtensions)) {
            echo "PFP not allowed.";
            exit;
        }

        //generate user id and store the profile picture
        $userCounter = readUserCounter($jsonFileUserCounter);
        $userCounter++;

        $userFileName = $uploadsUserProfilePath . '/' . $userCounter . '.gif';
        move_uploaded_file($pfp['tmp_name'], $userFileName);

        //hash password and store user information
        $hashedPassword = password_hash($password, PASSWORD_BCRYPT);
        
        $users = readUsers($jsonFileUsers);
        $newUser = [
            'user_id' => $userCounter,
            'username' => $username,
            'password' => $hashedPassword,
            'pfp' => $userFileName,
            'mod' => 0
        ];

        $users[] = $newUser;
        writeUsers($jsonFileUsers, $users);
        writeUserCounter($jsonFileUserCounter, $userCounter);

        //redirect to a confirmation page with the user ID and username
        header("Location: signup_confirmation.php?user_id={$userCounter}&username=" . urlencode($username));
        exit;
		
    } else { echo "complete all fields."; exit; }
}
?>

<html>
	<head>
		<link rel="stylesheet" href="CSS/style.css">
        <link rel="icon" type="image/x-icon" href="anostra.ico">
		<title>Signup - Anostra</title>
	</head>
	<body>
		<table>
			<tr>
				<td>
		            <form method="post" enctype="multipart/form-data">
			            <label for="username">Username (32 chars):</label>
			            <input type="text" name="username" maxlength="32" required>

			            <label for="password">Password:</label>
			            <input type="password" name="password" required>

			            <label for="pfp">PFP (GIF only, Max 8KB, Square recommended):</label>
			            <input type="file" name="pfp" accept="image/gif" required>

			            <input type="submit" value="Signup">
			
			            <p>Protip: use 100+ random ascii characters as a password and save it. Username needed to sign in.</p>
			
		            </form>
                </td>
            </tr>
        </table>
	</body>
</html>
